import jssc.*;

class AllCode
{
	static String retData = "";
	static Boolean done = false;
	static SerialPort serialPort;
	static int datacount = 0;


    public static void main (String[] args)
    {
		//Here is the code that will be executed

        //Assign the COM port number we wish to open
        serialPort = new SerialPort("COM32");

		//Open Port
		COM_Open();

		//Clear LCD
		LCDClear();

		//Print Program
		LCDPrint("Java Control", 0, 0);

		int loopCount = 4;

		while (loopCount > 0)
		{
			//Initialise Variable
			int Light = ReadLight();

			//While Light Sensor is reading above 1000
			while (Light > 1000)
			{
				//Collect Light Level and save to variable
				Light = ReadLight();

				//Send Commands to the FA Robot
				//Clear LCD
				LCDClear();

				//Print Light Level to LCD
				LCDPrint("Light: " + String.valueOf(Light), 0, 8);
			}

			PlayNote(100, 20);
			PlayNote(200, 20);
			PlayNote(400, 20);
			PlayNote(800, 20);
			PlayNote(400, 20);
			PlayNote(200, 20);
			PlayNote(100, 20);

			int count = 0;

			while (count < 4)
			{
				Forwards(10);
				Right(90);
				count++;
			}

			loopCount--;
		}

		//Close the port
		COM_Close();
    }








	//Port Functions
    private static void COM_Open()
    {
		try
        {
			//Open The Port
			System.out.println("Port Opened: " + serialPort.openPort());

			//Setup The Port
			System.out.println("Port Setup: " + serialPort.setParams(SerialPort.BAUDRATE_115200,
					SerialPort.DATABITS_8,
					SerialPort.STOPBITS_1,
					SerialPort.PARITY_NONE));

            int mask = SerialPort.MASK_RXCHAR;
            //Set the prepared mask
            serialPort.setEventsMask(mask);
            //Add an interface through which we will receive information about events
            serialPort.addEventListener(new SerialPortReader());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

    private static void COM_Close()
    {
		try
        {
			serialPort.closePort();;
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static int GetAPIVersion()
	{
		int retval = 0;
		try
		{
			serialPort.writeBytes("GetAPIVersion;".getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}


	//LED & Speaker Functions
    private static void LEDWrite(int value)
    {
		String dataOut = "LEDWrite " + value + ";";

		try
        {
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

    private static void LEDOn(int index)
    {
		String dataOut = "LEDOn " + index + ";";

		try
        {
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

    private static void LEDOff(int index)
    {
		String dataOut = "LEDOff " + index + ";";

		try
        {
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

    private static void PlayNote(int note, int time)
    {
		String dataOut = "PlayNote " + note + " " + time + ";";

		try
        {
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}

		try
		{
		    Thread.sleep(time);
		}
		catch(InterruptedException ex)
		{
		    Thread.currentThread().interrupt();
		}
	}


	//Motor Functions
    private static void SetMotors(int left, int right)
    {
		String dataOut = "SetMotors " + left + " " + right + ";";

		try
        {
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

    private static void Forwards(int distance)
    {
		String dataOut = "Forwards " + distance + ";";

		try
        {
			serialPort.writeBytes(dataOut.getBytes());
			ConvertInDataToInt(1);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

    private static void Backwards(int distance)
    {
		String dataOut = "Backwards " + distance + ";";

		try
        {
			serialPort.writeBytes(dataOut.getBytes());
			ConvertInDataToInt(1);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

    private static void Left(int angle)
    {
		String dataOut = "Left " + angle + ";";

		try
        {
			serialPort.writeBytes(dataOut.getBytes());
			ConvertInDataToInt(1);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

    private static void Right(int angle)
    {
		String dataOut = "Right " + angle + ";";

		try
        {
			serialPort.writeBytes(dataOut.getBytes());
			ConvertInDataToInt(1);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}


	//LCD Functions
    private static void LCDClear()
    {
		try
        {
			serialPort.writeBytes("LCDClear;".getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void LCDPrint(String data, int x, int y)
	{
		String dataOut = "LCDPrint " + x + " " + y + " " + data + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void LCDNumber(int data, int x, int y)
	{
		String dataOut = "LCDNumber " + x + " " + y + " " + data + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void LCDPixel(int state, int x, int y)
	{
		String dataOut = "LCDPixel " + x + " " + y + " " + state + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void LCDLine(int x1, int y1, int x2, int y2)
	{
		String dataOut = "LCDLine " + x1 + " " + y1 + " " + x2 + " " + y2 + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void LCDRect(int x1, int y1, int x2, int y2)
	{
		String dataOut = "LCDRect " + x1 + " " + y1 + " " + x2 + " " + y2 + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void LCDBacklight(int value)
	{
		String dataOut = "LCDBacklight " + value + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void LCDVerbose(int value)
	{
		String dataOut = "LCDVerbose " + value + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void LCDOptions(int foreground, int background, int transparent)
	{
		String dataOut = "LCDBacklight " + foreground + " " + background + " " + transparent + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}


	//Sensor Functions
	private static int ReadSwitch(int index)
	{
		int retval = 0;
		try
		{
			String dataOut = "ReadSwitch " + index + ";";
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(1);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int ReadIR(int index)
	{
		int retval = 0;
		try
		{
			String dataOut = "ReadIR " + String.valueOf(index) + ";";
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(4);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int ReadLine(int index)
	{
		int retval = 0;
		try
		{
			String dataOut = "ReadLine " + String.valueOf(index) + ";";
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(4);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int ReadLight()
	{
		int retval = 0;
		try
		{
			serialPort.writeBytes("ReadLight;".getBytes());
			retval = ConvertInDataToInt(4);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int ReadMic()
	{
		int retval = 0;
		try
		{
			serialPort.writeBytes("ReadMic;".getBytes());
			retval = ConvertInDataToInt(4);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int ReadAxis(int index)
	{
		int retval = 0;
		try
		{
			String dataOut = "ReadAxis " + String.valueOf(index) + ";";
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(5);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}


	//Servo Motor Functions
	private static void ServoEnable(int index)
	{
		String dataOut = "ServoEnable " + String.valueOf(index) + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void ServoDisable(int index)
	{
		String dataOut = "ServoDisable " + String.valueOf(index) + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void ServoSetPos(int index, int position)
	{
		String dataOut = "ServoSetPos " + index + " " + position + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void ServoAutoMove(int index, int position)
	{
		String dataOut = "ServoAutoMove " + index + " " + position + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}

	private static void ServoMoveSpeed(int speed)
	{
		String dataOut = "ServoMoveSpeed " + String.valueOf(speed) + ";";

		try
		{
			serialPort.writeBytes(dataOut.getBytes());
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
	}


	//SD Card Functions
	private static int CardInit()
	{
		int retval = 255;
		try
		{
			serialPort.writeBytes("CardInit;".getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int CardCreate(String filename)
	{
		int retval = 255;
		String dataOut = "CardCreate " + filename + ";";
		try
		{
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int CardOpen(String filename)
	{
		int retval = 255;
		String dataOut = "CardOpen " + filename + ";";
		try
		{
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int CardDelete(String filename)
	{
		int retval = 255;
		String dataOut = "CardDelete " + filename + ";";
		try
		{
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int CardWriteByte(int data)
	{
		int retval = 255;
		String dataOut = "CardWriteByte " + data + ";";
		try
		{
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int CardReadByte()
	{
		int retval = 255;
		try
		{
			serialPort.writeBytes("CardReadByte;".getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int CardRecordMic(int bitdepth, int samplerate, int time, String filename)
	{
		int retval = 255;
		String dataOut = "CardRecordMic " + bitdepth + " " + samplerate + " " + time + " " + filename + ";";
		try
		{
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int CardPlayback(String filename)
	{
		int retval = 255;
		String dataOut = "CardPlayback " + filename + ";";
		try
		{
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}

	private static int CardBitmap(int x, int y, String filename)
	{
		int retval = 255;
		String dataOut = "CardBitmap " + x + " " + y + " " + filename + ";";
		try
		{
			serialPort.writeBytes(dataOut.getBytes());
			retval = ConvertInDataToInt(3);
		}
		catch (SerialPortException ex)
		{
			System.out.println(ex);
		}
		return retval;
	}


	//Conversion Functions
	private static int ConvertInDataToInt(int numBytes)
	{
		int retval = 0;
		retData = "";
		done = false;
		datacount = numBytes;

		int timeout = 30000;

		while ((done == false) && (timeout > 0))
		{
			timeout--;
			try
			{
				Thread.sleep(100);
			}
			catch(InterruptedException ex)
			{
				Thread.currentThread().interrupt();
			}
		}

		if (timeout > 0)
		{
			try
			{
				String data = retData.trim();
				retval = Integer.parseInt(data);
			}
			catch (NumberFormatException ex)
			{
				System.out.println(ex);
				retval = 0;
			}
		}

		return retval;
	}


   static class SerialPortReader implements SerialPortEventListener
   {

        public void serialEvent(SerialPortEvent event)
        {
            //Object type SerialPortEvent carries information about which event occurred and a value.
            //For example, if the data came a method event.getEventValue() returns us the number of bytes in the input buffer.
            if(event.isRXCHAR())
            {
                if(event.getEventValue() > 0)
                {
					done = true;

                    try
                    {
                       retData = serialPort.readString(event.getEventValue());
                    }
                    catch (SerialPortException ex) {
                        System.out.println(ex);
                    }
                }
            }
        }
    }



}